import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/remedical.m.dart';
import 'package:fis_measure/values/unit_desc.dart';
import 'package:fis_measure/view/cursor.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/measure_configuration_style.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';
import 'package:fis_theme/theme.dart';

class PatternBodyItem extends FStatelessWidget {
  final String patternBodyItemTitle;
  final FWidget patternBodyItem;
  final bool? isEnd;
  const PatternBodyItem({
    Key? key,
    required this.patternBodyItemTitle,
    required this.patternBodyItem,
    this.isEnd = false,
  }) : super(key: key);
  @override
  FWidget build(BuildContext context) {
    return FColumn(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        FText(
          patternBodyItemTitle,
          style: const TextStyle(
            fontWeight: FontWeight.bold,
          ),
        ),
        const FSizedBox(
          height: 8,
        ),
        patternBodyItem,
        if (!isEnd!)
          FContainer(
            decoration: const BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  color: Color.fromRGBO(235, 235, 235, 1),
                  width: 1,
                ),
              ),
            ),
          ),
        const FSizedBox(
          height: 16,
        ),
      ],
    );
  }
}

class PatternItem extends FStatelessWidget {
  final String title;
  final FWidget item;
  const PatternItem({required this.title, required this.item, Key? key})
      : super(key: key);
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.only(bottom: 8),
      child: FRow(
        children: [
          FContainer(
            width: 200,
            child: FText(title),
          ),
          FExpanded(
            child: FContainer(
              alignment: Alignment.centerLeft,
              child: item,
            ),
          ),
          const FSizedBox(
            width: 20,
          ),
        ],
      ),
    );
  }
}

class PatternItemRadio extends FStatelessWidget {
  final List<dynamic> tabList;
  final PatternValueType? type;
  final dynamic value;
  final ValueCallback onChangeState;

  ///父级节点
  final FInteractiveContainer businessParent;
  const PatternItemRadio({
    Key? key,
    required this.businessParent,
    required this.tabList,
    required this.value,
    required this.onChangeState,
    this.type = PatternValueType.size,
  }) : super(key: key);

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.only(left: 10),
      child: FWrap(
        spacing: 8.0, // 主轴(水平)方向间距
        runSpacing: 8.0, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        crossAlignment: WrapCrossAlignment.center,
        alignment: WrapAlignment.start,
        children: List<FWidget>.generate(
          tabList.length,
          (index) {
            FWidget child = const FText("");
            switch (type) {
              case PatternValueType.size:
                child = FText(
                  MeasureTextSize(tabList[index]).toString(),
                  style: tabList[index] != value
                      ? const TextStyle(
                          color: Colors.black,
                        )
                      : const TextStyle(
                          color: Colors.white,
                        ),
                );
                break;
              case PatternValueType.unit:
                if (tabList is List<Unit>) {
                  child = FText(
                    UnitDescriptionMap.getDescByInt(tabList[index].index),
                    style: tabList[index] != value
                        ? const TextStyle(
                            color: Colors.black,
                          )
                        : const TextStyle(
                            color: Colors.white,
                          ),
                  );
                }
                break;
              case PatternValueType.cursor:
                child = MeasureCursor(
                  size: 10,
                  type: tabList[index],
                  color: tabList[index] != value ? Colors.black : Colors.white,
                );
                break;
              default:
                child = const FText("");
            }
            var tab = tabList[index];
            return FElevatedButton(
              name: "$tab",
              businessParent: businessParent,
              onPressed: () {
                if (tabList[index] != value) onChangeState.call(tabList[index]);
              },
              style: ButtonStyle(
                backgroundColor: tabList[index] != value
                    ? MaterialStateProperty.all(
                        Colors.white,
                      )
                    : MaterialStateProperty.all(
                        FTheme.ins.colorScheme.primary,
                      ),
              ),
              child: FContainer(
                width: getIconButtonWidth(type),
                height: 20,
                child: FCenter(
                  child: child,
                ),
              ),
            );
          },
        ),
      ),
    );
  }

  double getIconButtonWidth(PatternValueType? type) {
    switch (type) {
      case PatternValueType.size:
        return 50;
      case PatternValueType.unit:
        return 46;
      case PatternValueType.cursor:
        return 20;
      default:
        return 50;
    }
  }
}

enum PatternValueType {
  size,
  unit,
  cursor,
}

class PatternItemSwitch extends FStatelessWidget {
  const PatternItemSwitch({
    Key? key,
    required this.onChanged,
    required this.value,
  }) : super(key: key);
  final ValueCallback onChanged;
  final bool value;

  @override
  FWidget build(BuildContext context) {
    return FSwitch(
      inactiveThumbColor: FTheme.ins.colorScheme.primary,
      inactiveTrackColor: Colors.grey.withAlpha(88),
      activeColor: Colors.white,
      splashRadius: 0,
      activeTrackColor: FTheme.ins.colorScheme.primary,
      onChanged: (value) => onChanged.call(value),
      value: value,
    );
  }
}

class PatternItemSelect extends StatelessWidget implements FWidget {
  const PatternItemSelect({
    super.key,
    required this.onChanged,
    required this.value,
    required this.itemList,
  });
  final ValueCallback onChanged;
  final String value;
  final List<MeasureSelectModel> itemList;
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.symmetric(
        vertical: 4,
        horizontal: 10,
      ),
      child: FSelect<MeasureSelectModel, String>(
        source: itemList,
        fontFamily: FTheme.ins.localeSetting.fontFamily,
        hintText: i18nBook.measure.selectAnnotationFontSize.t,
        value: value,
        clearable: false,
        height: 36,
        width: 400,
        optionLabelExtractor: (data) => data.name,
        optionValueExtractor: (data) => data.value.toString(),
        onSelectChanged: (value, index) => onChanged.call(value),
      ),
    );
  }
}
